#include "regezusb.h"

void wait_1ms(void)
{
	DPS = 0;
/*   Delay for 1 millisecond (1000 microseconds). */
/*   10 cycles * 166.6 ns per cycle is 1.66 microseconds per loop. */
/*   1000 microseconds / 1.66 = 602.  [assumes 24 MHz clock] */
               
	_asm
        mov     dptr,#(0xffff - 602)    ; long pulse for operating
        mov     r4,#5
    loop:
        inc     dptr            ; 3 cycles
        mov     a,dpl           ; 2 cycles
        orl     a,dph           ; 2 cycles
        jnz     loop            ; 3 cycles
        _endasm;
}


#pragma SAVE
#pragma NOOVERLAY

void int0_isr(void) interrupt 0
{
	_asm
        push _DPS
        _endasm;
	DPS = 0;
	/* clear interrupt */
	_asm
	clr _TCON+1
        _endasm;


	
	_asm
	pop _DPS
        _endasm;
}

void timer0_isr(void) interrupt 1
{
	static char div;

	_asm
        push _DPS
        _endasm;
	DPS = 0;
	/* clear interrupt */
	_asm
	clr _TCON+5
        _endasm;

	div++;
	div &= 0x07;
	if (!div)
		OUTC ^= 0x06;
	
	_asm
	pop _DPS
        _endasm;
}

void int1_isr(void) interrupt 2
{
	_asm
        push _DPS
        _endasm;
	DPS = 0;
	/* clear interrupt */
	_asm
	clr _TCON+3
        _endasm;


	
	_asm
	pop _DPS
        _endasm;
}

void timer1_isr(void) interrupt 3
{
	_asm
        push _DPS
        _endasm;
	DPS = 0;
	/* clear interrupt */
	_asm
	clr _TCON+7
        _endasm;


	
	_asm
	pop _DPS
        _endasm;
}

void ser0_isr(void) interrupt 4
{
	_asm
        push _DPS
        _endasm;
	DPS = 0;




	/* clear interrupt */
	_asm
	clr _SCON0+0
	clr _SCON0+1
        _endasm;
	/* epilogue */
	_asm
	pop _DPS
        _endasm;
}

void timer2_isr(void) interrupt 5
{
	_asm
        push _DPS
        _endasm;
	DPS = 0;
	/* clear interrupt */
	_asm
	clr _T2CON+7
        _endasm;


	
	_asm
	pop _DPS
        _endasm;
}

void resume_isr(void) interrupt 6
{
	_asm
        push _DPS
        _endasm;
	DPS = 0;
	/* clear interrupt */
	_asm
	clr _EICON+4
        _endasm;


	
	_asm
	pop _DPS
        _endasm;
}

void ser1_isr(void) interrupt 7
{
	_asm
        push _DPS
        _endasm;
	DPS = 0;


	
	/* clear interrupt */
	_asm
	clr _SCON1+0
	clr _SCON1+1
        _endasm;
	/* epilogue */
	_asm
	pop _DPS
        _endasm;
}

void i2c_isr(void) interrupt 9
{
	_asm
        push _DPS
        _endasm;
	DPS = 0;
	/* clear interrupt */
	_asm
	mov a,_EXIF
        clr acc.5
        mov _EXIF,a
        _endasm;


	
	_asm
	pop _DPS
        _endasm;
}

void int4_isr(void) interrupt 10
{
	_asm
        push _DPS
        _endasm;
	DPS = 0;
	/* clear interrupt */
	_asm
	mov a,_EXIF
        clr acc.6
        mov _EXIF,a
        _endasm;


	
	_asm
	pop _DPS
        _endasm;
}

void int5_isr(void) interrupt 11
{
	_asm
        push _DPS
        _endasm;
	DPS = 0;
	/* clear interrupt */
	_asm
	mov a,_EXIF
        clr acc.7
        mov _EXIF,a
        _endasm;


	
	_asm
	pop _DPS
        _endasm;
}

void int6_isr(void) interrupt 12
{
	_asm
        push _DPS
        _endasm;
	DPS = 0;
	/* clear interrupt */
	_asm
	clr _EICON+3
        _endasm;

	
	_asm
	pop _DPS
        _endasm;
}

static const char usb_device_descriptor[] = { 
	0x12,        /* bLength */
	0x01,        /* bDescriptorType */
	0x00, 0x01,  /* bcdUSB */
	0xff,        /* bDeviceClass (vendor specific) */
	0x00,        /* bDeviceSubClass */
	0xff,        /* bDeviceProtocol (vendor specific) */
	0x40,        /* bMaxPacketSize0 */
	0x34, 0x12,  /* idVendor */
	0x78, 0x56,  /* idProduct */
	0x02, 0x01,  /* bcdDevice */
	0x00,        /* iManufacturer */
	0x00,        /* iProduct */
	0x00,        /* iSerialNumber */
	0x01         /* bNumConfigurations */
};

static const char usb_config0_descriptor[] = {
	0x09,        /* bLength */
	0x02,        /* bDescriptorType */
	0x19, 0x00,  /* wTotalLength */
	0x01,        /* bNumInterfaces */
	0x01,        /* bConfigurationValue */
	0x00,        /* iConfiguration */
	0x40,        /* bmAttributs */
	0x19,        /* MaxPower */
	/* interface descriptor */
	0x09,        /* bLength */
	0x04,        /* bDescriptorType */
	0x00,        /* bInterfaceNumber */
	0x00,        /* bAlternateSetting */
	0x01,        /* bNumEndpoints */
	0xff,        /* bInterfaceClass (vendor specific) */
	0x00,        /* bInterfaceSubClass */
	0xff,        /* bInterfaceProtocol (vendor specific) */
	0x00,        /* iInterface */
	/* endpoint descriptor */
	0x07,        /* bLength */
	0x05,        /* bDescriptorType */
	0x02,        /* bEndpointAddress */
	0x02,        /* bmAttributes (bulk) */
	0x40, 0x00,  /* wMaxPacketSize */
	0x00         /* bInterval */
};

void usb_sudav_isr(void) interrupt 13
{
	char tmp;

	_asm
        push _DPS
        _endasm;
	DPS = 0;
	/* clear interrupt */
	_asm
	mov a,_EXIF
        clr acc.4
        mov _EXIF,a
        _endasm;
        USBIRQ = 0x01;
	/* process control transfer */
	switch (SETUPDAT[1]) { /* bRequest field */
	case 0x00:  /* get status */
		switch (SETUPDAT[0]) {  /* bmRequest field */
		case 0x80:  /* IN, device */
			IN0BUF[0] = 1; /* self powered, no remote wakeup */
			IN0BUF[1] = 0;
			IN0BC = 2;
			goto setupack;

		case 0x82:  /* IN, endpoint */
			tmp = SETUPDAT[4];
			if (tmp >= 1 && tmp <= 7)
				IN0BUF[0] = ((xdata char *)(&OUT1CS))[2*(tmp-1)];
			else if  (tmp >= 0x81 && tmp <= 0x87)
				IN0BUF[0] = ((xdata char *)(&IN1CS))[2*(tmp-0x81)];
			else
				IN0BUF[0] = 0;
			IN0BUF[1] = 0;
			IN0BC = 2;
			goto setupack;

		case 0x81:  /* IN, interface */
			IN0BUF[0] = 0;
			IN0BUF[1] = 0;
			IN0BC = 2;
			goto setupack;
		}
		goto setupstall;

	case 0x01:  /* clear feature */
		switch (SETUPDAT[0]) {  /* bmRequest field */
		case 0x00:  /* OUT, device */
			/* remote wakeup */
			goto setupack;

		case 0x02:  /* OUT, endpoint */
			tmp = SETUPDAT[4];
			if (tmp >= 1 && tmp <= 7)
				((xdata char *)(&OUT1CS))[2*(tmp-1)] = 0; /* load val into byte cnt register */
			else if  (tmp >= 0x81 && tmp <= 0x87)
				((xdata char *)(&IN1CS))[2*(tmp-0x81)] = 0; /* should clear busy */
			else
				goto setupstall;
			/* should clear data toggle */
			goto setupack;
		}
		goto setupstall;
		
	case 0x03:  /* set feature */
		switch (SETUPDAT[0]) {  /* bmRequest field */
		case 0x00:  /* OUT, device */
			/* remote wakeup */
			goto setupack;

		case 0x02:  /* OUT, endpoint */
			tmp = SETUPDAT[4];
			if (tmp >= 1 && tmp <= 7)
				((xdata char *)(&OUT1CS))[2*(tmp-1)] = 1; /* load val into byte cnt register */
			else if  (tmp >= 0x81 && tmp <= 0x87)
				((xdata char *)(&IN1CS))[2*(tmp-0x81)] = 1; /* should clear busy */
			else
				goto setupstall;
			/* should clear data toggle */
			goto setupack;
		}
		goto setupstall;

	case 0x06:  /* get descriptor */
		if (SETUPDAT[0] != 0x80)
			goto setupstall;
		if (SETUPDAT[3] == 0x01) {
			SUDPTRH = ((unsigned int)usb_device_descriptor) >> 8;
			SUDPTRL = ((unsigned int)usb_device_descriptor);
			goto setupack;
		}
		if (SETUPDAT[3] == 0x02 && SETUPDAT[2] == 0) {
			SUDPTRH = ((unsigned int)usb_config0_descriptor) >> 8;
			SUDPTRL = ((unsigned int)usb_config0_descriptor);
			goto setupack;
		}
		goto setupstall;

	case 0x07:  /* set descriptor */
		goto setupstall;

	case 0x08:  /* get configuration */
		goto setupstall;
		
	case 0x09:  /* set configuration */
		if (SETUPDAT[0] != 0x00)
			goto setupstall;
		if (SETUPDAT[2] != 0x01)
			goto setupstall;
		goto setupack;

	case 0x0a:  /* get interface */
	case 0x0b:  /* set interface */
	case 0x0c:  /* sync frame */
	default:
		goto setupstall;
	}
  setupack:
	EP0CS = 2;  /* clear HSNAK */
	goto endusbisr;
  setupstall:
	EP0CS = 3;  /* signal stall */
	goto endusbisr;
	
  endusbisr:

	
	_asm
	pop _DPS
        _endasm;
}

void usb_sof_isr(void) interrupt 14
{
	_asm
        push _DPS
        _endasm;
	DPS = 0;
	/* clear interrupt */
	_asm
	mov a,_EXIF
        clr acc.4
        mov _EXIF,a
        _endasm;
	USBIRQ = 0x02;


	
	_asm
	pop _DPS
        _endasm;
}

void usb_sutok_isr(void) interrupt 15
{
	_asm
        push _DPS
        _endasm;
	DPS = 0;
	/* clear interrupt */
	_asm
	mov a,_EXIF
        clr acc.4
        mov _EXIF,a
        _endasm;
	USBIRQ = 0x04;


	
	_asm
	pop _DPS
        _endasm;
}

void usb_suspend_isr(void) interrupt 16
{
	_asm
        push _DPS
        _endasm;
	DPS = 0;
	/* clear interrupt */
	_asm
	mov a,_EXIF
        clr acc.4
        mov _EXIF,a
        _endasm;
	USBIRQ = 0x08;


	
	_asm
	pop _DPS
        _endasm;
}

void usb_usbreset_isr(void) interrupt 17
{
	_asm
        push _DPS
        _endasm;
	DPS = 0;
	/* clear interrupt */
	_asm
	mov a,_EXIF
        clr acc.4
        mov _EXIF,a
        _endasm;
	USBIRQ = 0x10;


	
	_asm
	pop _DPS
        _endasm;
}

void usb_ep0in_isr(void) interrupt 18
{
	_asm
        push _DPS
        _endasm;
	DPS = 0;
	/* clear interrupt */
	_asm
	mov a,_EXIF
        clr acc.4
        mov _EXIF,a
        _endasm;
	IN07IRQ = 0x01;

	
	_asm
	pop _DPS
        _endasm;
}

void usb_ep0out_isr(void) interrupt 19
{
	_asm
        push _DPS
        _endasm;
	DPS = 0;
	/* clear interrupt */
	_asm
	mov a,_EXIF
        clr acc.4
        mov _EXIF,a
        _endasm;
	OUT07IRQ = 0x01;


	
	_asm
	pop _DPS
        _endasm;
}

void usb_ep1in_isr(void) interrupt 20
{
	_asm
        push _DPS
        _endasm;
	DPS = 0;
	/* clear interrupt */
	_asm
	mov a,_EXIF
        clr acc.4
        mov _EXIF,a
        _endasm;
	IN07IRQ = 0x02;


	
	_asm
	pop _DPS
        _endasm;
}

void usb_ep1out_isr(void) interrupt 21
{
	_asm
        push _DPS
        _endasm;
	DPS = 0;
	/* clear interrupt */
	_asm
	mov a,_EXIF
        clr acc.4
        mov _EXIF,a
        _endasm;
	OUT07IRQ = 0x02;


	
	_asm
	pop _DPS
        _endasm;
}

void usb_ep2in_isr(void) interrupt 22
{
	_asm
        push _DPS
        _endasm;
	DPS = 0;
	/* clear interrupt */
	_asm
	mov a,_EXIF
        clr acc.4
        mov _EXIF,a
        _endasm;
	IN07IRQ = 0x04;


	
	_asm
	pop _DPS
        _endasm;
}

void usb_ep2out_isr(void) interrupt 23
{
	_asm
        push _DPS
        _endasm;
	DPS = 0;
	/* clear interrupt */
	_asm
	mov a,_EXIF
        clr acc.4
        mov _EXIF,a
        _endasm;
	OUT07IRQ = 0x04;


	
	_asm
	pop _DPS
        _endasm;
}

void usb_ep3in_isr(void) interrupt 24
{
	_asm
        push _DPS
        _endasm;
	DPS = 0;
	/* clear interrupt */
	_asm
	mov a,_EXIF
        clr acc.4
        mov _EXIF,a
        _endasm;
	IN07IRQ = 0x08;


	
	_asm
	pop _DPS
        _endasm;
}

void usb_ep3out_isr(void) interrupt 25
{
	_asm
        push _DPS
        _endasm;
	DPS = 0;
	/* clear interrupt */
	_asm
	mov a,_EXIF
        clr acc.4
        mov _EXIF,a
        _endasm;
	OUT07IRQ = 0x08;


	
	_asm
	pop _DPS
        _endasm;
}

void usb_ep4in_isr(void) interrupt 26
{
	_asm
        push _DPS
        _endasm;
	DPS = 0;
	/* clear interrupt */
	_asm
	mov a,_EXIF
        clr acc.4
        mov _EXIF,a
        _endasm;
	IN07IRQ = 0x10;


	
	_asm
	pop _DPS
        _endasm;
}

void usb_ep4out_isr(void) interrupt 27
{
	_asm
        push _DPS
        _endasm;
	DPS = 0;
	/* clear interrupt */
	_asm
	mov a,_EXIF
        clr acc.4
        mov _EXIF,a
        _endasm;
	OUT07IRQ = 0x10;


	
	_asm
	pop _DPS
        _endasm;
}

void usb_ep5in_isr(void) interrupt 28
{
	_asm
        push _DPS
        _endasm;
	DPS = 0;
	/* clear interrupt */
	_asm
	mov a,_EXIF
        clr acc.4
        mov _EXIF,a
        _endasm;
	IN07IRQ = 0x20;


	
	_asm
	pop _DPS
        _endasm;
}

void usb_ep5out_isr(void) interrupt 29
{
	_asm
        push _DPS
        _endasm;
	DPS = 0;
	/* clear interrupt */
	_asm
	mov a,_EXIF
        clr acc.4
        mov _EXIF,a
        _endasm;
	OUT07IRQ = 0x20;


	
	_asm
	pop _DPS
        _endasm;
}

void usb_ep6in_isr(void) interrupt 30
{
	_asm
        push _DPS
        _endasm;
	DPS = 0;
	/* clear interrupt */
	_asm
	mov a,_EXIF
        clr acc.4
        mov _EXIF,a
        _endasm;
	IN07IRQ = 0x40;


	
	_asm
	pop _DPS
        _endasm;
}

void usb_ep6out_isr(void) interrupt 31
{
	_asm
        push _DPS
        _endasm;
	DPS = 0;
	/* clear interrupt */
	_asm
	mov a,_EXIF
        clr acc.4
        mov _EXIF,a
        _endasm;
	OUT07IRQ = 0x40;


	
	_asm
	pop _DPS
        _endasm;
}

void usb_ep7in_isr(void) interrupt 32
{
	_asm
        push _DPS
        _endasm;
	DPS = 0;
	/* clear interrupt */
	_asm
	mov a,_EXIF
        clr acc.4
        mov _EXIF,a
        _endasm;
	IN07IRQ = 0x80;


	
	_asm
	pop _DPS
        _endasm;
}

void usb_ep7out_isr(void) interrupt 33
{
	_asm
        push _DPS
        _endasm;
	DPS = 0;
	/* clear interrupt */
	_asm
	mov a,_EXIF
        clr acc.4
        mov _EXIF,a
        _endasm;
	OUT07IRQ = 0x80;


	
	_asm
	pop _DPS
        _endasm;
}



#pragma RESTORE

//void main(void){}

void ezusbmain(void)
{
	unsigned int mscntr;

	ISOCTL = 1;  /* disable isochronous endpoints */
	USBBAV = 1;  /* enable autovector, disable breakpoint logic */
	/*
	 * Timer setup:
	 * timer 0: LED blink    : T0M=0, CT0=0, MODE0=1
	 * timer 1: Baudrate gen : T1M=1, CT1=0, MODE1=2
	 */
	CKCON = 0x31; /* one external wait state, to avoid chip bugs */
	TMOD = 0x21;
	TCON = 0x55;  /* INT0/INT1 edge */
	TH1 = 256-156;  /* 1200 bauds */
	PCON = 0;  /* SMOD0=0 */

	PORTCCFG = 0;
	OUTC = 0x04;
	OEC = 0x86;

        PORTBCFG = 0;
	OUTB = 0x00;
	OEB = 0x02;

	/* enable interrupts */
	IE = 0x82;  /* enable timer 0 int */
	EIE = 0x01; /* enable USB interrupts */
        USBIEN = 1; /* enable SUDAV interrupt */

        USBCS = 0;  /* disconnect from USB */
        for (mscntr = 0; mscntr < 1500; mscntr++)
                wait_1ms();
        USBCS = 2;  /* 8051 handles control */
        USBCS = 6;  /* reconnect, 8051 handles control */
	for (;;);
}

